@@ -12,23 +12,37 @@ import errno
 import codecs
 import signal
 import urllib
+import logging
 import datetime
 import optparse
 import tempfile
 import ConfigParser
-
 from operator import itemgetter
 
+import gtimelog
+
+log = logging.getLogger('gtimelog')
+
 # Which Gnome toolkit should we use?  Prior to 0.7, pygtk was the default with
 # a fallback to gi (gobject introspection), except on Ubuntu where gi was
 # forced.  With 0.7, gi was made the default in upstream, so the Ubuntu
 # specific patch isn't necessary.
-try:
-    import gi
-    toolkit = 'gi'
-except ImportError:
-    import pygtk
-    toolkit = 'pygtk'
+
+if '--prefer-pygtk' in sys.argv:
+    sys.argv.remove('--prefer-pygtk')
+    try:
+        import pygtk
+        toolkit = 'pygtk'
+    except ImportError:
+        import gi
+        toolkit = 'gi'
+else:
+    try:
+        import gi
+        toolkit = 'gi'
+    except ImportError:
+        import pygtk
+        toolkit = 'pygtk'
 
 
 if toolkit == 'gi':
@@ -46,6 +60,11 @@ if toolkit == 'gi':
     gtk_status_icon_new = gtk.StatusIcon.new_from_file
     pango_tabarray_new = pango.TabArray.new
 
+    if gtk._version.startswith('2'):
+        gtk_version = 2
+    else:
+        gtk_version = 3
+
     try:
         if gtk._version.startswith('2'):
             from gi.repository import AppIndicator
@@ -64,6 +83,7 @@ else:
     from gtk import gdk as gdk
     import pango
 
+    gtk_version = 2
     PANGO_ALIGN_LEFT = pango.TAB_LEFT
     GTK_RESPONSE_OK = gtk.RESPONSE_OK
     gtk_status_icon_new = gtk.status_icon_new_from_file
@@ -784,9 +804,33 @@ class TimeLog(object):
         self.virtual_midnight = virtual_midnight
         self.reread()
 
+    def virtual_today(self):
+        """Return today's date, adjusted for virtual midnight."""
+        return virtual_day(datetime.datetime.now(), self.virtual_midnight)
+
+    def check_reload(self):
+        """Look at the mtime of timelog.txt, and reload it if necessary.
+
+        Returns True if the file was reloaded.
+        """
+        mtime = self.get_mtime()
+        if mtime != self.last_mtime:
+            self.reread()
+            return True
+        else:
+            return False
+
+    def get_mtime(self):
+        """Return the mtime of self.filename, or None if the file doesn't exist."""
+        try:
+            return os.stat(self.filename).st_mtime
+        except OSError:
+            return None
+
     def reread(self):
         """Reload today's log."""
-        self.day = virtual_day(datetime.datetime.now(), self.virtual_midnight)
+        self.last_mtime = self.get_mtime()
+        self.day = self.virtual_today()
         min = datetime.datetime.combine(self.day, self.virtual_midnight)
         max = min + datetime.timedelta(1)
         self.history = []
@@ -794,10 +838,41 @@ class TimeLog(object):
                                  self.virtual_midnight,
                                  callback=self.history.append)
         self.need_space = not self.window.items
+        self._cache = {(min, max): self.window}
 
     def window_for(self, min, max):
         """Return a TimeWindow for a specified time interval."""
-        return TimeWindow(self.filename, min, max, self.virtual_midnight)
+        try:
+            return self._cache[min, max]
+        except KeyError:
+            window = TimeWindow(self.filename, min, max, self.virtual_midnight)
+            if len(self._cache) > 1000:
+                self._cache.clear()
+            self._cache[min, max] = window
+            return window
+
+    def window_for_day(self, date):
+        """Return a TimeWindow for the specified day."""
+        min = datetime.datetime.combine(date, self.virtual_midnight)
+        max = min + datetime.timedelta(1)
+        return self.window_for(min, max)
+
+    def window_for_week(self, date):
+        """Return a TimeWindow for the week that contains date."""
+        monday = date - datetime.timedelta(date.weekday())
+        min = datetime.datetime.combine(monday, self.virtual_midnight)
+        max = min + datetime.timedelta(7)
+        return self.window_for(min, max)
+
+    def window_for_month(self, date):
+        """Return a TimeWindow for the month that contains date."""
+        first_of_this_month = first_of_month(date)
+        first_of_next_month = next_month(date)
+        min = datetime.datetime.combine(
+            first_of_this_month, self.virtual_midnight)
+        max = datetime.datetime.combine(
+            first_of_next_month, self.virtual_midnight)
+        return self.window_for(min, max)
 
     def whole_history(self):
         """Return a TimeWindow for the whole history."""
@@ -962,6 +1037,8 @@ class Settings(object):
     mailer = 'x-terminal-emulator -e "mutt -H %s"'
     spreadsheet = 'xdg-open %s'
     chronological = True
+    summary_view = False
+    show_tasks = True
 
     enable_gtk_completion = True  # False enables gvim-style completion
 
@@ -995,6 +1072,8 @@ class Settings(object):
         config.set('gtimelog', 'mailer', self.mailer)
         config.set('gtimelog', 'spreadsheet', self.spreadsheet)
         config.set('gtimelog', 'chronological', str(self.chronological))
+        config.set('gtimelog', 'summary_view', str(self.summary_view))
+        config.set('gtimelog', 'show_tasks', str(self.show_tasks))
         config.set('gtimelog', 'gtk-completion',
                    str(self.enable_gtk_completion))
         config.set('gtimelog', 'hours', str(self.hours))
@@ -1020,6 +1099,8 @@ class Settings(object):
         self.mailer = config.get('gtimelog', 'mailer')
         self.spreadsheet = config.get('gtimelog', 'spreadsheet')
         self.chronological = config.getboolean('gtimelog', 'chronological')
+        self.summary_view = config.getboolean('gtimelog', 'summary_view')
+        self.show_tasks = config.getboolean('gtimelog', 'show_tasks')
         self.enable_gtk_completion = config.getboolean('gtimelog',
                                                        'gtk-completion')
         self.hours = config.getfloat('gtimelog', 'hours')
@@ -1054,17 +1135,19 @@ class IconChooser:
         # not necessarily the case!  this logic works for, say,
         # Ambiance/Radiance, but it gets New Wave and Dark Room wrong.
         if toolkit == 'gi':
-            style = gtk.MenuBar().get_style_context()
+            menu_bar = gtk.MenuBar()
+            # need to hold a reference to menu_bar to avoid LP#1016212
+            style = menu_bar.get_style_context()
             color = style.get_color(gtk.StateFlags.NORMAL)
             value = (color.red + color.green + color.blue) / 3
         else:
             style = gtk.MenuBar().rc_get_style()
             color = style.text[gtk.STATE_NORMAL]
             value = color.value
-        if value >= 0.5:
-            return icon_file_bright
-        else:
-            return icon_file_dark
+        filename = icon_file_bright if value >= 0.5 else icon_file_dark
+        log.debug('Menu bar color: (%g, %g, %g), averages to %g; picking %s',
+                  color.red, color.green, color.blue, value, filename)
+        return filename
 
 
 class SimpleStatusIcon(IconChooser):
@@ -1082,11 +1165,13 @@ class SimpleStatusIcon(IconChooser):
         self.tick()
         self.icon.connect('activate', self.on_activate)
         self.icon.connect('popup-menu', self.on_popup_menu)
-        self.gtimelog_window.main_window.connect(
-            'style-set', self.on_style_set) # Gtk+ 2
-        self.gtimelog_window.main_window.connect(
-            'style-updated', self.on_style_set) # Gtk+ 3
-        gobject.timeout_add(1000, self.tick)
+        if gtk_version == 2:
+            self.gtimelog_window.main_window.connect(
+                'style-set', self.on_style_set)
+        else: # assume Gtk+ 3
+            self.gtimelog_window.main_window.connect(
+                'style-updated', self.on_style_set)
+        gobject.timeout_add_seconds(1, self.tick)
         self.gtimelog_window.entry_watchers.append(self.entry_added)
         self.gtimelog_window.tray_icon = self
 
@@ -1108,9 +1193,14 @@ class SimpleStatusIcon(IconChooser):
     def on_popup_menu(self, widget, button, activate_time):
         """The user clicked on the icon."""
         tray_icon_popup_menu = self.gtimelog_window.tray_icon_popup_menu
-        tray_icon_popup_menu.popup(
-            None, None, gtk.status_icon_position_menu,
-            button, activate_time, self.icon)
+        if toolkit == "gi":
+            tray_icon_popup_menu.popup(
+                None, None, gtk.StatusIcon.position_menu,
+                self.icon, button, activate_time)
+        else:
+            tray_icon_popup_menu.popup(
+                None, None, gtk.status_icon_position_menu,
+                button, activate_time, self.icon)
 
     def entry_added(self, entry):
         """An entry has been added."""
@@ -1141,6 +1231,10 @@ class SimpleStatusIcon(IconChooser):
 class AppIndicator(IconChooser):
     """Ubuntu's application indicator for gtimelog."""
 
+    # XXX: on Ubuntu 10.04 the app indicator apparently doesn't understand
+    # set_icon('/absolute/path'), and so gtimelog ends up being without an
+    # icon.  I don't know if I want to continue supporting Ubuntu 10.04.
+
     def __init__(self, gtimelog_window):
         self.gtimelog_window = gtimelog_window
         self.timelog = gtimelog_window.timelog
@@ -1152,10 +1246,12 @@ class AppIndicator(IconChooser):
         self.indicator.set_status(APPINDICATOR_ACTIVE)
         self.indicator.set_menu(gtimelog_window.app_indicator_menu)
         self.gtimelog_window.tray_icon = self
-        self.gtimelog_window.main_window.connect(
-            'style-set', self.on_style_set) # Gtk+ 2
-        self.gtimelog_window.main_window.connect(
-            'style-updated', self.on_style_set) # Gtk+ 3
+        if gtk_version == 2:
+            self.gtimelog_window.main_window.connect(
+                'style-set', self.on_style_set)
+        else: # assume Gtk+ 3
+            self.gtimelog_window.main_window.connect(
+                'style-updated', self.on_style_set)
 
     def available(self):
         """Is the icon supported by this system?
@@ -1198,15 +1294,17 @@ class OldTrayIcon(IconChooser):
         self.last_tick = False
         self.tick(force_update=True)
         self.trayicon.show_all()
-        self.gtimelog_window.main_window.connect(
-            'style-set', self.on_style_set) # Gtk+ 2
-        self.gtimelog_window.main_window.connect(
-            'style-updated', self.on_style_set) # Gtk+ 3
+        if gtk_version == 2:
+            self.gtimelog_window.main_window.connect(
+                'style-set', self.on_style_set)
+        else: # assume Gtk+ 3
+            self.gtimelog_window.main_window.connect(
+                'style-updated', self.on_style_set)
         tray_icon_popup_menu = gtimelog_window.tray_icon_popup_menu
         self.eventbox.connect_object(
             'button-press-event', self.on_press, tray_icon_popup_menu)
         self.eventbox.connect('button-release-event', self.on_release)
-        gobject.timeout_add(1000, self.tick)
+        gobject.timeout_add_seconds(1, self.tick)
         self.gtimelog_window.entry_watchers.append(self.entry_added)
         self.gtimelog_window.tray_icon = self
 
@@ -1235,6 +1333,8 @@ class OldTrayIcon(IconChooser):
         else:
             self.gtimelog_window.tray_show.show()
             self.gtimelog_window.tray_hide.hide()
+        # I'm assuming toolkit == 'pygtk' here, since there's now way the old
+        # EggTrayIcon can work with PyGI/Gtk+ 3.
         widget.popup(None, None, None, event.button, event.time)
 
     def on_release(self, widget, event):
@@ -1281,10 +1381,6 @@ class OldTrayIcon(IconChooser):
 class MainWindow:
     """Main application window."""
 
-    # Initial view mode.
-    chronological = True
-    show_tasks = True
-
     # URL to use for Help -> Online Documentation.
     help_url = "http://mg.pov.lt/gtimelog"
 
@@ -1299,7 +1395,10 @@ class MainWindow:
         # Try to prevent timer routines mucking with the buffer while we're
         # mucking with the buffer.  Not sure if it is necessary.
         self.lock = False
-        self.chronological = settings.chronological
+        self.chronological = settings.chronological and not settings.summary_view
+        self.summary_view = settings.summary_view
+        self.show_tasks = settings.show_tasks
+        self.looking_at_date = None
         self.entry_watchers = []
         self._init_ui()
 
@@ -1310,6 +1409,8 @@ class MainWindow:
         # Set initial state of menu items *before* we hook up signals
         chronological_menu_item = builder.get_object('chronological')
         chronological_menu_item.set_active(self.chronological)
+        summary_menu_item = builder.get_object('summary')
+        summary_menu_item.set_active(self.summary_view)
         show_task_pane_item = builder.get_object('show_task_pane')
         show_task_pane_item.set_active(self.show_tasks)
         # Now hook up signals.
@@ -1334,6 +1435,7 @@ class MainWindow:
             self.on_calendar_day_selected_double_click)
         self.main_window = builder.get_object('main_window')
         self.main_window.connect('delete_event', self.delete_event)
+        self.current_view_label = builder.get_object('current_view_label')
         self.log_view = builder.get_object('log_view')
         self.set_up_log_view_columns()
         self.task_pane = builder.get_object('task_list_pane')
@@ -1375,13 +1477,12 @@ class MainWindow:
         self.populate_log()
         self.update_show_checkbox()
         self.tick(True)
-        gobject.timeout_add(1000, self.tick)
+        gobject.timeout_add_seconds(1, self.tick)
 
     def set_up_log_view_columns(self):
         """Set up tab stops in the log view."""
         # we can't get a Pango context for unrealized widgets
-        if not self.log_view.get_realized():
-            self.log_view.realize()
+        self.log_view.realize()
         pango_context = self.log_view.get_pango_context()
         em = pango_context.get_font_description().get_size()
         tabs = pango_tabarray_new(2, False)
@@ -1405,15 +1506,26 @@ class MainWindow:
         if self.footer_mark is not None:
             buffer.delete_mark(self.footer_mark)
             self.footer_mark = None
-        today = virtual_day(
-            datetime.datetime.now(), self.timelog.virtual_midnight)
-        today = today.strftime('%A, %Y-%m-%d (week %V)')
-        self.w(today + '\n\n', 'today')
-        if self.chronological:
-            for item in self.timelog.window.all_entries():
-                self.write_item(item)
+        if self.looking_at_date is None:
+            today = self.timelog.day
+            window = self.timelog.window
         else:
-            work, slack = self.timelog.window.grouped_entries()
+            today = self.looking_at_date
+            window = self.timelog.window_for_day(today)
+        today = today.strftime('%A, %Y-%m-%d (week %V)')
+        self.current_view_label.set_text(today)
+        if self.chronological:
+            for item in window.all_entries():
+                self.write_item(item)
+        elif self.summary_view:
+            entries, totals = window.categorized_work_entries()
+            for category, duration in sorted(totals.items()):
+                self.write_group(category or 'no category', duration)
+            where = buffer.get_end_iter()
+            where.backward_cursor_position()
+            buffer.place_cursor(where)
+        else:
+            work, slack = window.grouped_entries()
             for start, entry, duration in work + slack:
                 self.write_group(entry, duration)
             where = buffer.get_end_iter()
@@ -1434,8 +1546,9 @@ class MainWindow:
         buffer = self.log_buffer
         self.footer_mark = buffer.create_mark(
             'footer', buffer.get_end_iter(), True)
-        total_work, total_slacking = self.timelog.window.totals()
-        weekly_window = self.weekly_window()
+        window = self.daily_window(self.looking_at_date)
+        total_work, total_slacking = window.totals()
+        weekly_window = self.weekly_window(self.looking_at_date)
         week_total_work, week_total_slacking = weekly_window.totals()
         work_days_this_week = weekly_window.count_days()
 
@@ -1462,7 +1575,11 @@ class MainWindow:
             self.w(format_duration(per_diem), 'duration')
             self.w(' per day')
         self.w(')\n')
-        time_left = self.time_left_at_work(total_work)
+
+        if self.looking_at_date is None:
+            time_left = self.time_left_at_work(total_work)
+        else:
+            time_left = None
         if time_left is not None:
             time_to_leave = datetime.datetime.now() + time_left
             if time_left < datetime.timedelta(0):
@@ -1473,7 +1590,7 @@ class MainWindow:
             self.w(time_to_leave.strftime('%H:%M'), 'time')
             self.w(')')
 
-        if self.settings.show_office_hours:
+        if self.settings.show_office_hours and self.looking_at_date is None:
             self.w('\nAt office today: ')
             hours = datetime.timedelta(hours=self.settings.hours)
             total = total_slacking + total_work
@@ -1576,6 +1693,17 @@ class MainWindow:
         if entry not in [row[0] for row in self.completion_choices]:
             self.completion_choices.append([entry])
 
+    def jump_to_date(self, date):
+        """Switch to looking at a given date"""
+        if self.looking_at_date == date:
+            return
+        self.looking_at_date = date
+        self.populate_log()
+
+    def jump_to_today(self):
+        """Switch to looking at today"""
+        self.jump_to_date(None)
+
     def delete_event(self, widget, data=None):
         """Try to close the window."""
         if self.tray_icon:
@@ -1623,6 +1751,24 @@ class MainWindow:
         else:
             self.on_show_activate()
 
+    def on_today_toolbutton_clicked(self, widget=None):
+        """Toolbar: Back"""
+        self.jump_to_today()
+
+    def on_back_toolbutton_clicked(self, widget=None):
+        """Toolbar: Back"""
+        day = (self.looking_at_date or self.timelog.day)
+        self.jump_to_date(day - datetime.timedelta(1))
+
+    def on_forward_toolbutton_clicked(self, widget=None):
+        """Toolbar: Forward"""
+        day = (self.looking_at_date or self.timelog.day)
+        day += datetime.timedelta(1)
+        if day >= self.timelog.virtual_today():
+            self.jump_to_today()
+        else:
+            self.jump_to_date(day)
+
     def on_quit_activate(self, widget):
         """File -> Quit selected"""
         gtk.main_quit()
@@ -1639,13 +1785,26 @@ class MainWindow:
     def on_chronological_activate(self, widget):
         """View -> Chronological"""
         self.chronological = True
+        self.summary_view = False
         self.populate_log()
 
     def on_grouped_activate(self, widget):
         """View -> Grouped"""
         self.chronological = False
+        self.summary_view = False
         self.populate_log()
 
+    def on_summary_activate(self, widget):
+        """View -> Summary"""
+        self.chronological = False
+        self.summary_view = True
+        self.populate_log()
+
+    def daily_window(self, day=None):
+        if not day:
+            day = self.timelog.day
+        return self.timelog.window_for_day(day)
+
     def on_daily_report_activate(self, widget):
         """File -> Daily Report"""
         reports = Reports(self.timelog.window)
@@ -1653,19 +1812,15 @@ class MainWindow:
 
     def on_yesterdays_report_activate(self, widget):
         """File -> Daily Report for Yesterday"""
-        max = self.timelog.window.min_timestamp
-        min = max - datetime.timedelta(1)
-        reports = Reports(self.timelog.window_for(min, max))
+        day = self.timelog.day - datetime.timedelta(1)
+        reports = Reports(self.timelog.window_for_day(day))
         self.mail(reports.daily_report)
 
     def on_previous_day_report_activate(self, widget):
         """File -> Daily Report for a Previous Day"""
         day = self.choose_date()
         if day:
-            min = datetime.datetime.combine(
-                day, self.timelog.virtual_midnight)
-            max = min + datetime.timedelta(1)
-            reports = Reports(self.timelog.window_for(min, max))
+            reports = Reports(self.timelog.window_for_day(day))
             self.mail(reports.daily_report)
 
     def choose_date(self):
@@ -1688,12 +1843,7 @@ class MainWindow:
     def weekly_window(self, day=None):
         if not day:
             day = self.timelog.day
-        monday = day - datetime.timedelta(day.weekday())
-        min = datetime.datetime.combine(monday,
-                        self.timelog.virtual_midnight)
-        max = min + datetime.timedelta(7)
-        window = self.timelog.window_for(min, max)
-        return window
+        return self.timelog.window_for_week(day)
 
     def on_weekly_report_activate(self, widget):
         """File -> Weekly Report"""
@@ -1735,14 +1885,7 @@ class MainWindow:
     def monthly_window(self, day=None):
         if not day:
             day = self.timelog.day
-        first_of_this_month = first_of_month(day)
-        first_of_next_month = next_month(day)
-        min = datetime.datetime.combine(
-            first_of_this_month, self.timelog.virtual_midnight)
-        max = datetime.datetime.combine(
-            first_of_next_month, self.timelog.virtual_midnight)
-        window = self.timelog.window_for(min, max)
-        return window
+        return self.timelog.window_for_month(day)
 
     def on_previous_month_report_activate(self, widget):
         """File -> Monthly Report for a Previous Month"""
@@ -1829,6 +1972,8 @@ class MainWindow:
             self.task_pane.hide()
         else:
             self.task_pane.show()
+            if self.tasks.check_reload():
+                self.set_up_task_list()
 
     def on_task_pane_close_button_activate(self, event, data=None):
         """The close button next to the task pane title"""
@@ -1850,7 +1995,10 @@ class MainWindow:
 
     def task_list_button_press(self, menu, event):
         if event.button == 3:
-            menu.popup(None, None, None, event.button, event.time)
+            if toolkit == "gi":
+                menu.popup(None, None, None, None, event.button, event.time)
+            else:
+                menu.popup(None, None, None, event.button, event.time)
             return True
         else:
             return False
@@ -1927,6 +2075,9 @@ class MainWindow:
 
     def add_entry(self, widget, data=None):
         """Add the task entry to the log."""
+        if self.looking_at_date is not None:
+            self.jump_to_today()
+
         entry = self.task_entry.get_text()
         if not isinstance(entry, unicode):
             entry = unicode(entry, 'UTF-8')
@@ -1956,8 +2107,10 @@ class MainWindow:
         if not entry:
             return
         self.add_history(entry)
+        previous_day = self.timelog.day
         self.timelog.append(entry, now)
-        if self.chronological:
+        same_day = self.timelog.day == previous_day
+        if self.chronological and same_day:
             self.delete_footer()
             self.write_item(self.timelog.window.last_entry())
             self.add_footer()
@@ -1972,11 +2125,16 @@ class MainWindow:
 
     def tick(self, force_update=False):
         """Tick every second."""
-        if self.tasks.check_reload():
-            self.set_up_task_list()
+        if self.timelog.check_reload():
+            self.populate_log()
+            self.set_up_history()
+        if self.task_pane.get_property('visible'):
+            if self.tasks.check_reload():
+                self.set_up_task_list()
         now = datetime.datetime.now().replace(second=0, microsecond=0)
         if now == self.last_tick and not force_update:
-            # Do not eat CPU unnecessarily.
+            # Do not eat CPU unnecessarily: update the time ticker only when
+            # the minute changes.
             return True
         self.last_tick = now
         last_time = self.timelog.window.last_time()
@@ -1985,7 +2143,7 @@ class MainWindow:
         else:
             self.time_label.set_text(format_duration(now - last_time))
             # Update "time left to work"
-            if not self.lock:
+            if not self.lock and self.looking_at_date is None:
                 self.delete_footer()
                 self.add_footer()
         return True
@@ -2021,20 +2179,40 @@ if dbus:
 
 def main():
     """Run the program."""
-    parser = optparse.OptionParser(usage='%prog [options]')
-    parser.add_option('--sample-config', action='store_true',
-        help="write a sample configuration file to 'gtimelogrc.sample'")
-    parser.add_option('--ignore-dbus', action='store_true',
-        help="do not check if GTimeLog is already running")
-    parser.add_option('--replace', action='store_true',
-        help="replace the already running GTimeLog instance")
-    parser.add_option('--toggle', action='store_true',
-        help="show/hide the GTimeLog window if already running")
+    parser = optparse.OptionParser(usage='%prog [options]',
+                                   version=gtimelog.__version__)
     parser.add_option('--tray', action='store_true',
         help="start minimized")
+    parser.add_option('--sample-config', action='store_true',
+        help="write a sample configuration file to 'gtimelogrc.sample'")
+
+    dbus_options = optparse.OptionGroup(parser, "Single-Instance Options")
+    dbus_options.add_option('--replace', action='store_true',
+        help="replace the already running GTimeLog instance")
+    dbus_options.add_option('--quit', action='store_true',
+        help="tell an already-running GTimeLog instance to quit")
+    dbus_options.add_option('--toggle', action='store_true',
+        help="show/hide the GTimeLog window if already running")
+    dbus_options.add_option('--ignore-dbus', action='store_true',
+        help="do not check if GTimeLog is already running"
+             " (allows you to have multiple instances running)")
+    parser.add_option_group(dbus_options)
+
+    debug_options = optparse.OptionGroup(parser, "Debugging Options")
+    debug_options.add_option('--debug', action='store_true',
+        help="show debug information")
+    debug_options.add_option('--prefer-pygtk', action='store_true',
+        help="try to use the (obsolete) pygtk library instead of pygi")
+    parser.add_option_group(debug_options)
 
     opts, args = parser.parse_args()
 
+    log.addHandler(logging.StreamHandler(sys.stdout))
+    if opts.debug:
+        log.setLevel(logging.DEBUG)
+    else:
+        log.setLevel(logging.INFO)
+
     if opts.sample_config:
         settings = Settings()
         settings.save("gtimelogrc.sample")
@@ -2042,8 +2220,16 @@ def main():
         print "Edit it and save as %s" % settings.get_config_file()
         return
 
+    global dbus
+
+    if opts.debug:
+        print 'GTimeLog version: %s' % gtimelog.__version__
+        print 'Toolkit: %s' % toolkit
+        print 'Gtk+ version: %s' % gtk_version
+        print 'D-Bus available: %s' % ('yes' if dbus else 'no')
+        print 'GTimeLog directory: %s' % Settings().get_config_dir()
+
     if opts.ignore_dbus:
-        global dbus
         dbus = None
 
     # Let's check if there is already an instance of GTimeLog running
@@ -2055,9 +2241,11 @@ def main():
         try:
             session_bus = dbus.SessionBus()
             dbus_service = session_bus.get_object(SERVICE, OBJECT_PATH)
-            if opts.replace:
+            if opts.replace or opts.quit:
                 print 'gtimelog: Telling the already-running instance to quit'
                 dbus_service.Quit()
+                if opts.quit:
+                    sys.exit()
             elif opts.toggle:
                 dbus_service.ToggleFocus()
                 print 'gtimelog: Already running, toggling visibility'
@@ -2072,9 +2260,14 @@ def main():
         except dbus.DBusException, e:
             if e.get_dbus_name() == 'org.freedesktop.DBus.Error.ServiceUnknown':
                 # gtimelog is not running: that's fine and not an error at all
-                pass
+                if opts.quit:
+                    print 'gtimelog is not running'
+                    sys.exit()
             else:
                 sys.exit('gtimelog: %s' % e)
+    else: # not dbus
+        if opts.quit or opts.replace or opts.toggle:
+            sys.exit("gtimelog: dbus not available")
 
     settings = Settings()
     configdir = settings.get_config_dir()
@@ -2087,15 +2280,27 @@ def main():
             raise
     settings_file = settings.get_config_file()
     if not os.path.exists(settings_file):
+        if opts.debug:
+            print 'Saving settings to %s' % settings_file
         settings.save(settings_file)
     else:
+        if opts.debug:
+            print 'Loading settings from %s' % settings_file
         settings.load(settings_file)
+    if opts.debug:
+        print 'Loading time log from %s' % os.path.join(configdir, 'timelog.txt')
+        print 'Assuming date changes at %s' % settings.virtual_midnight
     timelog = TimeLog(os.path.join(configdir, 'timelog.txt'),
                       settings.virtual_midnight)
     if settings.task_list_url:
+        if opts.debug:
+            print 'Loading cached remote tasks from %s' % (
+                               os.path.join(configdir, 'remote-tasks.txt'))
         tasks = RemoteTaskList(settings.task_list_url,
                                os.path.join(configdir, 'remote-tasks.txt'))
     else:
+        if opts.debug:
+            print 'Loading tasks from %s' % os.path.join(configdir, 'tasks.txt')
         tasks = TaskList(os.path.join(configdir, 'tasks.txt'))
     main_window = MainWindow(timelog, settings, tasks)
     start_in_tray = False
@@ -2106,15 +2311,26 @@ def main():
             icons = [OldTrayIcon, SimpleStatusIcon, AppIndicator]
         else:
             icons = [SimpleStatusIcon, OldTrayIcon, AppIndicator]
+        if opts.debug:
+            print 'Tray icon preference: %s' % ', '.join(icon_class.__name__
+                                                         for icon_class in icons)
         for icon_class in icons:
             tray_icon = icon_class(main_window)
             if tray_icon.available():
+                if opts.debug:
+                    print 'Tray icon: %s' % icon_class.__name__
                 start_in_tray = (settings.start_in_tray
                                  if settings.start_in_tray
                                  else opts.tray)
                 break # found one that works
+            else:
+                if opts.debug:
+                    print '%s not available' % icon_class.__name__
     if not start_in_tray:
         main_window.on_show_activate()
+    else:
+        if opts.debug:
+            print 'Starting minimized'
     if dbus:
         service = Service(main_window)
     # This is needed to make ^C terminate gtimelog when we're using
